package org.seasar.cayenne.dao.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectstyle.cayenne.dba.TypesMapping;
import org.objectstyle.cayenne.map.DbAttribute;
import org.objectstyle.cayenne.map.DbEntity;
import org.objectstyle.cayenne.map.ObjAttribute;
import org.objectstyle.cayenne.map.ObjEntity;
import org.seasar.cayenne.S2DataContextFactory;
import org.seasar.cayenne.dao.CayenneDtoMetaData;
import org.seasar.extension.jdbc.PropertyType;

/**
 * @author Takahiro MATSUNAGA
 * @see AbstractSelectCommand
 */
public class SelectDtoCommand extends AbstractSelectCommand {

    /**
     * @param s2DataContextFactory
     * @param beanClass
     */
    public SelectDtoCommand(S2DataContextFactory s2DataContextFactory, Class beanClass) {
        super(s2DataContextFactory, beanClass);
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractSelectCommand#setupWhere(java.lang.StringBuffer, org.objectstyle.cayenne.map.ObjEntity, java.lang.Object[])
     */
    protected void setupWhere(StringBuffer sb, ObjEntity objEntity, Object[] args) {
        sb.append(" #chain('and' 'where')");

        DbEntity dbEntity = objEntity.getDbEntity();
        CayenneDtoMetaData dmd = new CayenneDtoMetaDataImpl(args[0].getClass());
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);
            String name = objAttr.getName();

            if (dmd.hasPropertyType(name)) {
                PropertyType pt = dmd.getPropertyType(name);

                sb.append(" #chunk($");
                sb.append(name).append(") ");
                sb.append(name).append(" #bindEqual($");
                sb.append(name).append(" '");
                sb.append(TypesMapping.getSqlNameByType(dbAttr.getType())).append("') #end");
            }
        }
        sb.append(" #end ");
        if (getQuery() != null) {
            sb.append(getQuery());
        }
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractSelectCommand#setupParams(java.lang.Object[])
     */
    protected void setupParams(Object[] args) {
        if (args.length == 0) {
            return;
        }

        Map params = new HashMap();
        CayenneDtoMetaData dmd = new CayenneDtoMetaDataImpl(args[0].getClass());
        for (int i = 0; i < dmd.getPropertyTypeSize(); ++i) {
            PropertyType pt = dmd.getPropertyType(i);
            params.put(pt.getColumnName(), pt.getPropertyDesc().getValue(args[0]));
        }

        setParams(params);
    }

}
