package org.seasar.cayenne.dao.impl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.objectstyle.cayenne.DataObject;
import org.objectstyle.cayenne.dba.TypesMapping;
import org.objectstyle.cayenne.map.DbAttribute;
import org.objectstyle.cayenne.map.DbEntity;
import org.objectstyle.cayenne.map.ObjAttribute;
import org.objectstyle.cayenne.map.ObjEntity;
import org.seasar.cayenne.S2DataContextFactory;
import org.seasar.framework.beans.BeanDesc;
import org.seasar.framework.beans.PropertyDesc;
import org.seasar.framework.beans.factory.BeanDescFactory;

/**
 * @author Takahiro MATSUNAGA
 * @see AbstractAutoCommand
 */
public class UpdateCommand extends AbstractAutoCommand {

    /**
     * @param S2DataContextFactory
     * @param beanClass
     */
    public UpdateCommand(S2DataContextFactory s2DataContextFactory, Class beanClass) {
        super(s2DataContextFactory, beanClass);
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractAutoCommand#setupSql(org.objectstyle.cayenne.map.ObjEntity)
     */
    protected void setupSql(ObjEntity objEntity) {
        StringBuffer sb = new StringBuffer("update ");
        sb.append(objEntity.getDbEntityName()).append(" set ");

        DbEntity dbEntity = objEntity.getDbEntity();
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            if (!dbAttr.isPrimaryKey()) {
                ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

                sb.append(objAttr.getName()).append(" = #bind($");
                sb.append(objAttr.getName()).append(" '");
                sb.append(TypesMapping.getSqlNameByType(dbAttr.getType())).append("'), ");
            }
        }
        sb.setLength(sb.length() - 2);
        setupWhere(sb, objEntity);
        
        setSql(sb.toString());
    }

    private void setupWhere(StringBuffer sb, ObjEntity objEntity) {
        sb.append(" #chain('and' 'where')");

        DbEntity dbEntity = objEntity.getDbEntity();
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            if (dbAttr.isPrimaryKey()) {
                ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

                sb.append(" #chunk($");
                sb.append(objAttr.getName()).append(") ");
                sb.append(objAttr.getName()).append(" #bindEqual($");
                sb.append(objAttr.getName()).append(" '");
                sb.append(TypesMapping.getSqlNameByType(dbAttr.getType())).append("') #end");
            }
        }
        sb.append(" #end");
    }

    /* (Javadoc Ȃ)
     * @see org.sersar.cayenne.dao.impl.AbstractAutoCommand#setupParams(org.objectstyle.cayenne.map.ObjEntity, org.objectstyle.cayenne.DataObject)
     */
    protected void setupParams(ObjEntity objEntity, DataObject dataObject) {
        Map params = new HashMap();

        BeanDesc beanDesc = BeanDescFactory.getBeanDesc(dataObject.getClass());

        DbEntity dbEntity = objEntity.getDbEntity();
        for (Iterator it = dbEntity.getAttributes().iterator(); it.hasNext();) {
            DbAttribute dbAttr = (DbAttribute) it.next();
            ObjAttribute objAttr = objEntity.getAttributeForDbAttribute(dbAttr);

            PropertyDesc propDesc = beanDesc.getPropertyDesc(objAttr.getName());
            params.put(objAttr.getName(), propDesc.getValue(dataObject));
        }

        setParams(params);
    }

}
